@@ -19,26 +19,19 @@
  */
 
 #include "file_sync.h"
+#include "filename_scape.h"
 #include <opensync/file.h>
 #include <opensync/opensync-version.h>
+#include <assert.h>
 #include <stdlib.h>
 
 static void free_dir(OSyncFileDir *dir)
 {
-	if (dir->path)
-		g_free(dir->path);
-		
-	if (dir->objtype)
-		g_free(dir->objtype);
-
-	if (dir->objformat)
-		g_free(dir->objformat);
-
 	if (dir->sink)
 		osync_objtype_sink_unref(dir->sink);
 
 	if (dir->hashtable)
-		osync_hashtable_free(dir->hashtable);
+		osync_hashtable_unref(dir->hashtable);
 
 	g_free(dir);
 }
@@ -56,135 +49,12 @@ static void free_env(OSyncFileEnv *env)
 	g_free(env);
 }
 
-static osync_bool osync_filesync_parse_directory(OSyncFileEnv *env, xmlNode *cur, OSyncError **error)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, cur, error);
-
-	OSyncFileDir *dir = osync_try_malloc0(sizeof(OSyncFileDir), error);
-	if (!dir)
-		goto error;
-	dir->env = env;
-	
-	while (cur != NULL) {
-		char *str = (char*)xmlNodeGetContent(cur);
-		if (str) {
-			if (!xmlStrcmp(cur->name, (const xmlChar *)"path")) {
-				dir->path = g_strdup(str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objtype")) {
-				dir->objtype = g_strdup(str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objformat")) {
-				dir->objformat = g_strdup(str);
-			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"recursive")) {
-				dir->recursive = (g_ascii_strcasecmp(str, "TRUE") == 0);
-			}
-			xmlFree(str);
-		}
-		cur = cur->next;
-	}
-
-	if (!dir->path) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Path not set");
-		goto error_free_dir;
-	}
-
-	osync_trace(TRACE_INTERNAL, "Got directory %s with objtype %s", dir->path, dir->objtype);
-
-	env->directories = g_list_append(env->directories, dir);
-
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return TRUE;
-
-error_free_dir:
-	free_dir(dir);
-error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return FALSE;
-}
-
-/*Load the state from a xml file and return it in the conn struct*/
-static osync_bool osync_filesync_parse_settings(OSyncFileEnv *env, const char *data, OSyncError **error)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, data, error);
-	xmlDoc *doc = NULL;
-	xmlNode *cur = NULL;
-
-	doc = xmlParseMemory(data, strlen(data) + 1);
-	if (!doc) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to parse settings");
-		goto error;
-	}
-
-	cur = xmlDocGetRootElement(doc);
-
-	if (!cur) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to get root element of the settings");
-		goto error_free_doc;
-	}
-
-	if (xmlStrcmp(cur->name, (xmlChar*)"config")) {
-		osync_error_set(error, OSYNC_ERROR_GENERIC, "Config valid is not valid");
-		goto error_free_doc;
-	}
-
-	cur = cur->xmlChildrenNode;
-
-	while (cur != NULL) {
-		char *str = (char*)xmlNodeGetContent(cur);
-		if (str) {
-			if (!xmlStrcmp(cur->name, (const xmlChar *)"directory")) {
-				if (!osync_filesync_parse_directory(env, cur->xmlChildrenNode, error))
-					goto error_free_doc;
-			}
-			xmlFree(str);
-		}
-		cur = cur->next;
-	}
-
-	xmlFreeDoc(doc);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return TRUE;
-
-error_free_doc:
-	xmlFreeDoc(doc);
-error:
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return FALSE;
-}
-
 static char *osync_filesync_generate_hash(struct stat *buf)
 {
 	char *hash = g_strdup_printf("%i-%i", (int)buf->st_mtime, (int)buf->st_ctime);
 	return hash;
 }
 
-static void osync_filesync_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
-{
-	OSyncError *error = NULL;
-	
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
-	
-	osync_context_report_success(ctx);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-	return;
-	
-error:
-	osync_context_report_osyncerror(ctx, error);
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-	osync_error_unref(&error);
-}
-
-static void osync_filesync_disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
-	
-	osync_context_report_success(ctx);
-	
-	osync_trace(TRACE_EXIT, "%s", __func__);
-}
-
-
 //typedef void (* OSyncSinkWriteFn) 
 //typedef void (* OSyncSinkCommittedAllFn) (void *data, OSyncPluginInfo *info, OSyncContext *ctx);
 
@@ -193,9 +63,8 @@ static osync_bool osync_filesync_read(void *userdata, OSyncPluginInfo *info, OSy
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, userdata, info, ctx, change);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
-	OSyncFileEnv *env = (OSyncFileEnv *)userdata;
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
 	OSyncError *error = NULL;
 	
 	char *filename = g_strdup_printf("%s/%s", dir->path, osync_change_get_uid(change));
@@ -212,48 +81,36 @@ static osync_bool osync_filesync_read(void *userdata, OSyncPluginInfo *info, OSy
 
 	OSyncData *odata = NULL;
 
-	if (strcmp("file", dir->objformat)) {
-
-		OSyncObjFormat *objformat = osync_format_env_find_objformat(formatenv, dir->objformat);
-		odata = osync_data_new(data, size, objformat, &error);
-		if (!odata) {
-			osync_change_unref(change);
-			osync_context_report_osyncwarning(ctx, error);
-			osync_error_unref(&error);
-			goto error_free_data;
-		}
-
-	} else {
-
-		OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
-		if (!file) {
-			osync_change_unref(change);
-			osync_context_report_osyncwarning(ctx, error);
-			osync_error_unref(&error);
-			goto error_free_data;
-		}
+	OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
+	if (!file) {
+		osync_change_unref(change);
+		osync_context_report_osyncwarning(ctx, error);
+		osync_error_unref(&error);
+		goto error_free_data;
+	}
 
 	
-		struct stat filestats;
-		stat(filename, &filestats);
-		file->userid = filestats.st_uid;
-		file->groupid = filestats.st_gid;
-		file->mode = filestats.st_mode;
-		file->last_mod = filestats.st_mtime;
+	struct stat filestats;
+	stat(filename, &filestats);
+	file->userid = filestats.st_uid;
+	file->groupid = filestats.st_gid;
+	file->mode = filestats.st_mode;
+	file->last_mod = filestats.st_mtime;
 
-		file->data = data;
-		file->size = size;
-		file->path = g_strdup(osync_change_get_uid(change));
-		
-		odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), env->objformat, &error);
-		if (!odata) {
-			osync_change_unref(change);
-			osync_context_report_osyncwarning(ctx, error);
-			osync_error_unref(&error);
-			g_free(file->path);
-			g_free(file);
-			goto error_free_data;
-		}
+	file->data = data;
+	file->size = size;
+	file->path = g_strdup(osync_change_get_uid(change));
+	
+	OSyncObjFormat *fileformat = osync_format_env_find_objformat(formatenv, "file");
+
+	odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), fileformat, &error);
+	if (!odata) {
+		osync_change_unref(change);
+		osync_context_report_osyncwarning(ctx, error);
+		osync_error_unref(&error);
+		g_free(file->path);
+		g_free(file);
+		goto error_free_data;
 	}
 
 	osync_data_set_objtype(odata, osync_objtype_sink_get_name(sink));
@@ -281,15 +138,22 @@ static osync_bool osync_filesync_write(void *data, OSyncPluginInfo *info, OSyncC
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, data, info, ctx, change);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
-	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
 	OSyncError *error = NULL;
 	OSyncData *odata = NULL;
 	char *buffer = NULL;
 	unsigned int size = 0;
 	
-	char *filename = g_strdup_printf ("%s/%s", dir->path, osync_change_get_uid(change));
-			
+	char *filename = NULL, *tmp = NULL;
+	if (!(tmp = strdup(osync_change_get_uid(change)))) {
+		osync_trace(TRACE_EXIT_ERROR, "%s", __func__);
+		return;
+	}
+	filename_scape_characters(tmp);
+
+	filename = g_strdup_printf ("%s/%s", dir->path, tmp);
+	free(tmp);
+
 	switch (osync_change_get_changetype(change)) {
 		case OSYNC_CHANGE_TYPE_DELETED:
 			if (!remove(filename) == 0) {
@@ -313,72 +177,13 @@ static osync_bool osync_filesync_write(void *data, OSyncPluginInfo *info, OSyncC
 			odata = osync_change_get_data(change);
 			g_assert(odata);
 
-			/* Convert to the configured store object format */
-			if (dir->objformat && strcmp("file", dir->objformat)) {
-
-				osync_bool ret;
-				OSyncFormatConverterPath *path = NULL;
-
-			        OSyncObjFormat *fileformat = osync_format_env_find_objformat(formatenv, "file");
-			        OSyncObjFormat *targetformat = osync_format_env_find_objformat(formatenv, dir->objformat);
-				OSyncObjFormat *detectedFormat = osync_format_env_detect_objformat_full(formatenv, odata, &error);
-				OSyncData *odata_fileformat = osync_data_clone(odata, &error);
-				osync_data_set_objformat(odata_fileformat, fileformat);
-				OSyncData *odata_detectedformat = osync_data_clone(odata, &error);
-				osync_data_set_objformat(odata_detectedformat, detectedFormat);
-
-				/* Sanity check - if the converters are disable the engine sends not the requested "file" object format */
-				if (fileformat == osync_data_get_objformat(odata)) {
-					/* Find converter path from file to detected format */
-					path = osync_format_env_find_path_with_detectors(formatenv, odata_fileformat, detectedFormat, &error);
-
-					if (!osync_format_env_convert(formatenv, path, odata, &error)) {
-						osync_error_set(&error, OSYNC_ERROR_EXISTS, "Can't convert to customized objformat.");
-						goto error;
-					}
-				}
-
-				/* Find converter path from detectedFormat to targetFromat.
-				   This is needed to avoid shortcuts path:
-				     "another object format with plain converter (or detector)" -> plain -> file
-				
-				   To be safe we convert $detectedFormat -> $targetformat in advance.
-				   And later convert $targetformat to fileFormat.
-				*/
-				path = osync_format_env_find_path_with_detectors(formatenv, odata_detectedformat, targetformat, &error);
-
-				if (!path)
-					goto error;
-
-				ret = osync_format_env_convert(formatenv, path, odata, &error);
-				osync_converter_path_unref(path);
-
-				if (!ret) {
-					osync_error_set(&error, OSYNC_ERROR_EXISTS, "Can't convert to customized objformat.");
-                			goto error;
-        			}
-
-
-				/* Find converter path fromat $targetformat to fileFormat. */
-				path = osync_format_env_find_path(formatenv, targetformat, fileformat, &error);
-
-				if (!path)
-					goto error;
-
-				ret = osync_format_env_convert(formatenv, path, odata, &error);
-				osync_converter_path_unref(path);
-
-				if (!ret) {
-					osync_error_set(&error, OSYNC_ERROR_EXISTS, "Can't convert to customized objformat.");
-                			goto error;
-        			}
-
-
-			}
-
 			osync_data_get_data(odata, &buffer, &size);
 			g_assert(buffer);
-			g_assert(size == sizeof(OSyncFileFormat));
+			if (size != sizeof(OSyncFileFormat)) {
+				osync_error_set(&error, OSYNC_ERROR_MISCONFIGURATION,
+					"The plugin file-sync only supports file format. Please re-configure and discover the plugin again.");
+				goto error;
+			}
 			
 			OSyncFileFormat *file = (OSyncFileFormat *)buffer;
 			
@@ -422,7 +227,7 @@ static void osync_filesync_report_dir(OSyncFileDir *directory, OSyncPluginInfo *
 	OSyncError *error = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, directory, subdir, ctx);
-	
+
 	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
 
 	path = g_build_filename(directory->path, subdir, NULL);
@@ -448,7 +253,6 @@ static void osync_filesync_report_dir(OSyncFileDir *directory, OSyncPluginInfo *
 		else
 			relative_filename = g_build_filename(subdir, de, NULL);
 			
-		osync_hashtable_report(directory->hashtable, relative_filename);
 		osync_trace(TRACE_INTERNAL, "path2 %s %s", filename, relative_filename);
 		
 		if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
@@ -459,35 +263,33 @@ static void osync_filesync_report_dir(OSyncFileDir *directory, OSyncPluginInfo *
 			
 			struct stat buf;
 			stat(filename, &buf);
-			char *hash = osync_filesync_generate_hash(&buf);
-			
-
-			OSyncChangeType type = osync_hashtable_get_changetype(directory->hashtable, relative_filename, hash);
-			if (type == OSYNC_CHANGE_TYPE_UNMODIFIED) {
-				g_free(hash);
-				g_free(filename);
-				g_free(relative_filename);
-				continue;
-			}
-			osync_hashtable_update_hash(directory->hashtable, type, relative_filename, hash);
 
 			/* Report normal files */
 			OSyncChange *change = osync_change_new(&error);
 			if (!change) {
 				osync_context_report_osyncwarning(ctx, error);
 				osync_error_unref(&error);
-				g_free(hash);
-				g_free(filename);
 				g_free(relative_filename);
 				continue;
 			}
-			
+
 			osync_change_set_uid(change, relative_filename);
+
+			char *hash = osync_filesync_generate_hash(&buf);
 			osync_change_set_hash(change, hash);
+			g_free(hash);
+
+			OSyncChangeType type = osync_hashtable_get_changetype(directory->hashtable, change);
 			osync_change_set_changetype(change, type);
 
-			g_free(hash);
-			
+			osync_hashtable_update_change(directory->hashtable, change);
+
+			if (type == OSYNC_CHANGE_TYPE_UNMODIFIED) {
+				g_free(filename);
+				g_free(relative_filename);
+				osync_change_unref(change);
+				continue;
+			}
 
 			char *data;
 			unsigned int size;
@@ -497,47 +299,38 @@ static void osync_filesync_report_dir(OSyncFileDir *directory, OSyncPluginInfo *
 				osync_context_report_osyncwarning(ctx, error);
 				osync_error_unref(&error);
 				g_free(filename);
+				g_free(relative_filename);
 				continue;
 			}
 
 			OSyncData *odata = NULL;
 
-			if (strcmp("file", directory->objformat)) {
+			OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
+			if (!file) {
+				osync_change_unref(change);
+				osync_context_report_osyncwarning(ctx, error);
+				osync_error_unref(&error);
+				g_free(filename);
+				g_free(relative_filename);
+				continue;
+			}
 
-				OSyncObjFormat *objformat = osync_format_env_find_objformat(formatenv, directory->objformat);
-				odata = osync_data_new(data, size, objformat, &error);
-				if (!odata) {
-					osync_change_unref(change);
-					osync_context_report_osyncwarning(ctx, error);
-					osync_error_unref(&error);
-					g_free(data);
-					continue;
-				}
+			file->data = data;
+			file->size = size;
+			file->path = g_strdup(relative_filename);
+			
+		        OSyncObjFormat *fileformat = osync_format_env_find_objformat(formatenv, "file");
 
-			} else {
-
-				OSyncFileFormat *file = osync_try_malloc0(sizeof(OSyncFileFormat), &error);
-				if (!file) {
-					osync_change_unref(change);
-					osync_context_report_osyncwarning(ctx, error);
-					osync_error_unref(&error);
-					g_free(filename);
-					g_free(relative_filename);
-					continue;
-				}
-
-				file->data = data;
-				file->size = size;
-				file->path = g_strdup(relative_filename);
-				
-				odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), directory->env->objformat, &error);
-				if (!odata) {
-					osync_change_unref(change);
-					osync_context_report_osyncwarning(ctx, error);
-					osync_error_unref(&error);
-					g_free(data);
-					continue;
-				}
+			odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), fileformat, &error);
+			if (!odata) {
+				osync_change_unref(change);
+				osync_context_report_osyncwarning(ctx, error);
+				osync_error_unref(&error);
+				g_free(data);
+				g_free(filename);
+				g_free(relative_filename);
+				g_free(file->path);
+				continue;
 			}
 
 			osync_data_set_objtype(odata, osync_objtype_sink_get_name(directory->sink));
@@ -565,35 +358,41 @@ static void osync_filesync_get_changes(void *data, OSyncPluginInfo *info, OSyncC
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
-	OSyncFileEnv *env = (OSyncFileEnv *)data;
-	int i = 0;
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
 	OSyncError *error = NULL;
 	
 	if (osync_objtype_sink_get_slowsync(dir->sink)) {
 		osync_trace(TRACE_INTERNAL, "Slow sync requested");
-		osync_hashtable_reset(dir->hashtable);
+		if (!osync_hashtable_slowsync(dir->hashtable, &error))
+		{
+			osync_context_report_osyncerror(ctx, error);
+			osync_trace(TRACE_EXIT_ERROR, "%s - %s", __func__, osync_error_print(&error));
+			osync_error_unref(&error);
+			return;
+		}
 	}
 	
 	osync_trace(TRACE_INTERNAL, "get_changes for %s", osync_objtype_sink_get_name(sink));
 
 	osync_filesync_report_dir(dir, info, NULL, ctx);
 	
-	char **uids = osync_hashtable_get_deleted(dir->hashtable);
-	for (i = 0; uids[i]; i++) {
+	OSyncList *u, *uids = osync_hashtable_get_deleted(dir->hashtable);
+	for (u = uids; u; u = u->next) {
 		OSyncChange *change = osync_change_new(&error);
 		if (!change) {
-			g_free(uids[i]);
 			osync_context_report_osyncwarning(ctx, error);
 			osync_error_unref(&error);
 			continue;
 		}
 		
-		osync_change_set_uid(change, uids[i]);
+		const char *uid = u->data;
+		osync_change_set_uid(change, uid);
 		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_DELETED);
 		
-		OSyncData *odata = osync_data_new(NULL, 0, env->objformat, &error);
+	        OSyncObjFormat *fileformat = osync_format_env_find_objformat(formatenv, "file");
+
+		OSyncData *odata = osync_data_new(NULL, 0, fileformat, &error);
 		if (!odata) {
-			g_free(uids[i]);
 			osync_change_unref(change);
 			osync_context_report_osyncwarning(ctx, error);
 			osync_error_unref(&error);
@@ -606,12 +405,11 @@ static void osync_filesync_get_changes(void *data, OSyncPluginInfo *info, OSyncC
 		
 		osync_context_report_change(ctx, change);
 		
-		osync_hashtable_update_hash(dir->hashtable, osync_change_get_changetype(change), osync_change_get_uid(change), NULL);
+		osync_hashtable_update_change(dir->hashtable, change);
 	
 		osync_change_unref(change);
-		g_free(uids[i]);
 	}
-	g_free(uids);
+	osync_list_free(uids);
 	
 	osync_context_report_success(ctx);
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -623,24 +421,31 @@ static void osync_filesync_commit_change(void *data, OSyncPluginInfo *info, OSyn
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
 	
-	char *filename = NULL;
+	char *filename = NULL, *tmp;
 	
 	if (!osync_filesync_write(data, info, ctx, change)) {
 		osync_trace(TRACE_EXIT_ERROR, "%s", __func__);
 		return;
 	}
-	
-	filename = g_strdup_printf ("%s/%s", dir->path, osync_change_get_uid(change));
+	if (!(tmp = strdup(osync_change_get_uid(change)))) {
+		osync_trace(TRACE_EXIT_ERROR, "%s", __func__);
+		return;
+	}
+	filename_scape_characters(tmp);
+
+	filename = g_strdup_printf ("%s/%s", dir->path, tmp);
+	free(tmp);
 	char *hash = NULL;
 	
 	if (osync_change_get_changetype(change) != OSYNC_CHANGE_TYPE_DELETED) {
 		struct stat buf;
 		stat(filename, &buf);
 		hash = osync_filesync_generate_hash(&buf);
+		osync_change_set_hash(change, hash);
 	}
 	g_free(filename);
 
-	osync_hashtable_update_hash(dir->hashtable, osync_change_get_changetype(change), osync_change_get_uid(change), hash);
+	osync_hashtable_update_change(dir->hashtable, change);
 	g_free(hash);
 	
 	osync_context_report_success(ctx);
@@ -651,6 +456,9 @@ static void osync_filesync_commit_change(void *data, OSyncPluginInfo *info, OSyn
 static void osync_filesync_sync_done(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+
+	OSyncError *error = NULL;
+
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
 
@@ -659,10 +467,20 @@ static void osync_filesync_sync_done(void *data, OSyncPluginInfo *info, OSyncCon
 	osync_anchor_update(anchorpath, path_field, dir->path);
 	g_free(anchorpath);
 	g_free(path_field);
+
+	if (!osync_hashtable_save(dir->hashtable, &error))
+		goto error;
 	
 	osync_context_report_success(ctx);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
+	return;
+
+error:
+	osync_context_report_osyncerror(ctx, error);
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
+	osync_error_unref(&error);
+	return;
 }
 
 /* In initialize, we get the config for the plugin. Here we also must register
@@ -675,40 +493,47 @@ static void *osync_filesync_initialize(OSyncPlugin *plugin, OSyncPluginInfo *inf
 	if (!env)
 		goto error;
 	
-	osync_trace(TRACE_INTERNAL, "The config: %s", osync_plugin_info_get_config(info));
-	
-	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
-	env->objformat = osync_format_env_find_objformat(formatenv, "file");
-	
-	if (!osync_filesync_parse_settings(env, osync_plugin_info_get_config(info), error))
-		goto error_free_env;
+	OSyncPluginConfig *config = osync_plugin_info_get_config(info);
+	assert(config);
 
-	/* Now we register the objtypes that we can sync. This plugin is special. It can
-	 * synchronize any objtype we configure it to sync and where a conversion
-	 * path to the file format can be found */
-	GList *o = env->directories;
-	for (; o; o = o->next) {
-		OSyncFileDir *dir = o->data;
-		/* We register the given objtype here */
-		OSyncObjTypeSink *sink = osync_objtype_sink_new(dir->objtype, error);
-		if (!sink)
+
+	GList *pathes = NULL;
+	int i, numobjs = osync_plugin_info_num_objtypes(info);
+	for (i = 0; i < numobjs; i++) {
+		OSyncFileDir *dir = osync_try_malloc0(sizeof(OSyncFileDir), error);
+		if (!dir)
 			goto error_free_env;
-		
-		if (!dir->objformat)
-			dir->objformat = g_strdup("file");
 
-		if (!osync_format_env_find_objformat(formatenv, dir->objformat)) {
-			osync_error_set(error, OSYNC_ERROR_GENERIC, "Configured storage format \"%s\" for object type \"%s\" is unknown. Is the format plugin missing?",
-					dir->objformat, dir->objtype);
-			osync_objtype_sink_unref(sink);
+		dir->sink = osync_plugin_info_nth_objtype(info, i);
+		assert(dir->sink);
+
+		const char *objtype = osync_objtype_sink_get_name(dir->sink);
+		OSyncPluginResource *res = osync_plugin_config_find_active_resource(config, objtype);
+		dir->path = osync_plugin_resource_get_path(res);
+		if (!dir->path) {
+			osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "Path for object type \"%s\" is not configured.", objtype);
 			goto error_free_env;
 		}
-	
+		if(g_list_find_custom(pathes, dir->path, (GCompareFunc)strcmp)) {
+			osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "Path for objtype \"%s\" defined for more than one objtype sink in configuration.", objtype);
+			goto error_free_env;
+		}
+		pathes = g_list_append(pathes, g_strdup(dir->path));
+
+		OSyncList *s = osync_plugin_resource_get_objformat_sinks(res);
+		for (; s; s = s->next) {
+			OSyncObjFormatSink *fsink = s->data;
+			const char *objformat = osync_objformat_sink_get_objformat(fsink);
+			assert(objformat);
+			
+			/* TODO: Implement objformat sanity check in OpenSync Core */ 
+			if (strcmp(objformat, "file")) {
+				osync_error_set(error, OSYNC_ERROR_MISCONFIGURATION, "Format \"%s\" is not supported by file-sync. Only Format \"file\" is currently supported by the file-sync plugin.", objformat);
+				goto error_free_env;
+			}
+
+		}
 
-		dir->sink = sink;
-		
-		osync_objtype_sink_add_objformat(sink, dir->objformat);
-		
 		/* All sinks have the same functions of course */
 		OSyncObjTypeSinkFunctions functions;
 		memset(&functions, 0, sizeof(functions));
@@ -720,19 +545,21 @@ static void *osync_filesync_initialize(OSyncPlugin *plugin, OSyncPluginInfo *inf
 		
 		/* We pass the OSyncFileDir object to the sink, so we dont have to look it up
 		 * again once the functions are called */
-		osync_objtype_sink_set_functions(sink, functions, dir);
-		osync_plugin_info_add_objtype(info, sink);
+		osync_objtype_sink_set_functions(dir->sink, functions, dir);
 
 		osync_trace(TRACE_INTERNAL, "The configdir: %s", osync_plugin_info_get_configdir(info));
 		char *tablepath = g_strdup_printf("%s/hashtable.db", osync_plugin_info_get_configdir(info));
-		dir->hashtable = osync_hashtable_new(tablepath, osync_objtype_sink_get_name(sink), error);
+		dir->hashtable = osync_hashtable_new(tablepath, objtype, error);
 		g_free(tablepath);
 	
 		if (!dir->hashtable)
 			goto error_free_env;
 
+		if (!osync_hashtable_load(dir->hashtable, error))
+			goto error_free_env;
+
 		char *anchorpath = g_strdup_printf("%s/anchor.db", osync_plugin_info_get_configdir(info));
-		char *path_field = g_strdup_printf("path_%s", osync_objtype_sink_get_name(sink));
+		char *path_field = g_strdup_printf("path_%s", osync_objtype_sink_get_name(dir->sink));
 		if (!osync_anchor_compare(anchorpath, path_field, dir->path))
 			osync_objtype_sink_set_slowsync(dir->sink, TRUE);
 		g_free(anchorpath);
@@ -744,6 +571,11 @@ static void *osync_filesync_initialize(OSyncPlugin *plugin, OSyncPluginInfo *inf
 		}
 	}
 
+	if (pathes) {
+		g_list_foreach(pathes, (GFunc)g_free, NULL);
+		g_list_free(pathes);
+	}
+
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, env);
 	return (void *)env;
 
@@ -762,16 +594,17 @@ static void osync_filesync_finalize(void *data)
 }
 
 /* Here we actually tell opensync which sinks are available. For this plugin, we
- * go through the list of directories and enable all, since all have been configured */
+ * just report all objtype as available. Since the resource are configured like this. */
 static osync_bool osync_filesync_discover(void *data, OSyncPluginInfo *info, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, error);
 	
-	OSyncFileEnv *env = (OSyncFileEnv *)data;
-	GList *o = env->directories;
-	for (; o; o = o->next) {
-		OSyncFileDir *dir = o->data;
-		osync_objtype_sink_set_available(dir->sink, TRUE);
+	int i, numobjs = osync_plugin_info_num_objtypes(info);
+	for (i = 0; i < numobjs; i++) {
+		OSyncObjTypeSink *sink = osync_plugin_info_nth_objtype(info, i);
+		g_assert(sink);
+
+		osync_objtype_sink_set_available(sink, TRUE);
 	}
 	
 	OSyncVersion *version = osync_version_new(error);
